@@ -1,3 +1,4 @@
+
 /* Copyright (C) 2004-2022 J.F.Dockes 
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -31,6 +32,7 @@
 #ifdef __FreeBSD__
 #include <osreldate.h>
 #endif
+#include <fnmatch.h>
 
 #include <algorithm>
 #include <iostream>
@@ -257,10 +259,10 @@ public:
     static std::string o_localecharset;
     // Limiting set of mime types to be processed. Normally empty.
     ParamStale    m_rmtstate;
-    std::unordered_set<std::string>   m_restrictMTypes; 
+    std::vector<std::string>   m_restrictMTypes; 
     // Exclusion set of mime types. Normally empty
     ParamStale    m_xmtstate;
-    std::unordered_set<std::string>   m_excludeMTypes; 
+    std::vector<std::string>   m_excludeMTypes; 
     // Metadata-gathering external commands (e.g. used to reap tagging info: "tmsu tags %f")
     ParamStale    m_mdrstate;
     std::vector<MDReaper> m_mdreapers;
@@ -480,17 +482,15 @@ RclConfig::RclConfig(const string *argcnf)
     m->zeroMe();
 
     if (m->o_origcwd.empty()) {
-        char buf[MAXPATHLEN];
-        if (getcwd(buf, MAXPATHLEN)) {
-            m->o_origcwd = string(buf);
-        } else {
+        m->o_origcwd = path_cwd();
+        if (m->o_origcwd.empty()) {
             std::cerr << "recollxx: can't retrieve current working "
                 "directory: relative path translations will fail\n";
         }
     }
 
-    // Compute our data dir name, typically /usr/local/share/recoll
-    m->m_datadir = path_pkgdatadir();
+    // Compute our data dir name, e.g. /usr/share/recoll
+    m->m_datadir = path_rclpkgdatadir();
     // We only do the automatic configuration creation thing for the default
     // config dir, not if it was specified through -c or RECOLL_CONFDIR
     bool autoconfdir = false;
@@ -541,7 +541,7 @@ RclConfig::RclConfig(const string *argcnf)
 #ifdef _WIN32
         m->o_localecharset = winACPName();
 #elif defined(__APPLE__)
-        m->o_localecharset = "UTF-8";
+        m->o_localecharset = cstr_utf8;
 #else
         const char *cp;
         cp = nl_langinfo(CODESET);
@@ -1124,7 +1124,7 @@ bool RclConfig::inStopSuffixes(const string& fni)
     SuffixStore::const_iterator it = m->m_stopsuffixes->find(fn);
     if (it != m->m_stopsuffixes->end()) {
         LOGDEB2("RclConfig::inStopSuffixes: Found (" << fni << ") ["  <<
-                ((*it).m->m_str) << "]\n");
+                ((*it).m_str) << "]\n");
         IdxDiags::theDiags().record(IdxDiags::NoContentSuffix, fni);
         return true;
     } else {
@@ -1201,27 +1201,35 @@ string RclConfig::getMimeHandlerDef(const string &mtype, bool filtertypes, const
     if (filtertypes) {
         if(m->m_rmtstate.needrecompute()) {
             m->m_restrictMTypes.clear();
-            stringToStrings(stringtolower((const string&)m->m_rmtstate.getvalue()),
-                            m->m_restrictMTypes);
+            stringToStrings(stringtolower(m->m_rmtstate.getvalue()), m->m_restrictMTypes);
         }
         if (m->m_xmtstate.needrecompute()) {
             m->m_excludeMTypes.clear();
-            stringToStrings(stringtolower((const string&)m->m_xmtstate.getvalue()),
-                            m->m_excludeMTypes);
+            stringToStrings(stringtolower(m->m_xmtstate.getvalue()), m->m_excludeMTypes);
         }
-        if (!m->m_restrictMTypes.empty() && !m->m_restrictMTypes.count(stringtolower(mtype))) {
-            IdxDiags::theDiags().record(IdxDiags::NotIncludedMime, fn, mtype);
-            LOGDEB1("RclConfig::getMimeHandlerDef: " << mtype << " not in mime type list\n");
-            return hs;
+        if (!m->m_restrictMTypes.empty()) {
+            bool inallowlist = false;
+            for (const auto& expr : m->m_restrictMTypes) {
+                if (fnmatch(expr.c_str(), mtype.c_str(), 0) == 0) {
+                    inallowlist = true;
+                }
+            }
+            if (!inallowlist) {
+                IdxDiags::theDiags().record(IdxDiags::NotIncludedMime, fn, mtype);
+                LOGERR("RclConfig::getMimeHandlerDef: " << mtype << " not in mime type list\n");
+                return hs;
+            }
         }
-        if (!m->m_excludeMTypes.empty() && m->m_excludeMTypes.count(stringtolower(mtype))) {
-            IdxDiags::theDiags().record(IdxDiags::ExcludedMime, fn, mtype);
-            LOGDEB1("RclConfig::getMimeHandlerDef: " << mtype << " in excluded mime list (fn " <<
-                    fn << ")\n");
-            return hs;
+        for (const auto& expr : m->m_excludeMTypes) {
+            if (fnmatch(expr.c_str(), mtype.c_str(), 0) == 0) {
+                IdxDiags::theDiags().record(IdxDiags::ExcludedMime, fn, mtype);
+                LOGERR("RclConfig::getMimeHandlerDef: " << mtype << " in excluded mime list (fn " <<
+                       fn << ")\n");
+                return hs;
+            }
         }
     }
-
+    
     if (!m->mimeconf->get(mtype, hs, "index")) {
         if (mtype.find("text/") == 0) {
             bool alltext{false};
@@ -1283,6 +1291,10 @@ bool RclConfig::getGuiFilter(const string& catfiltername, string& frag) const
 
 bool RclConfig::valueSplitAttributes(const string& whole, string& value, ConfSimple& attrs)
 {
+    if (whole.empty()) {
+        value.clear();
+        return false;
+    }
     bool inquote{false};
     string::size_type semicol0;    
     for (semicol0 = 0; semicol0 < whole.size(); semicol0++) {
@@ -1350,6 +1362,11 @@ bool RclConfig::Internal::readFieldsConfig(const string& cnferrloc)
     for (const auto& fieldname : tps) {
         string val;
         m_fields->get(fieldname, val, "prefixes");
+        // An empty value can be used to undo the default prefix and avoid indexing the field
+        if (val.empty()) {
+            LOGDEB0("Empty fields [prefixes] setting for " << fieldname << "\n");
+            continue;
+        }
         ConfSimple attrs;
         FieldTraits ft;
         // fieldname = prefix ; attr1=val;attr2=val...
@@ -1472,19 +1489,17 @@ bool RclConfig::Internal::readFieldsConfig(const string& cnferrloc)
 }
 
 // Return specifics for field name:
-bool RclConfig::getFieldTraits(const string& _fld, const FieldTraits **ftpp,
-                               bool isquery) const
+bool RclConfig::getFieldTraits(const string& _fld, const FieldTraits **ftpp, bool isquery) const
 {
     string fld = isquery ? fieldQCanon(_fld) : fieldCanon(_fld);
     map<string, FieldTraits>::const_iterator pit = m->m_fldtotraits.find(fld);
     if (pit != m->m_fldtotraits.end()) {
         *ftpp = &pit->second;
-        LOGDEB1("RclConfig::getFieldTraits: [" << _fld << "]->["  <<
-                pit->second.pfx << "]\n");
+        LOGDEB1("RclConfig::getFieldTraits: [" << _fld << "]->["  << pit->second.pfx << "]\n");
         return true;
     } else {
         LOGDEB1("RclConfig::getFieldTraits: no prefix for field [" << fld << "]\n");
-        *ftpp = 0;
+        *ftpp = nullptr;
         return false;
     }
 }
@@ -1812,7 +1827,7 @@ string RclConfig::getPidfile() const
             // being used by recollindex instances. So explicitely test for /run/user/$uid, still
             // leaving open the remote possibility that XDG_RUNTIME_DIR would be set to something
             // else...
-            rundir = path_cat("/run/user", lltodecstr(getuid()));
+            rundir = path_cat("/run/user", std::to_string(getuid()));
             if (path_isdir(rundir)) {
                 p = rundir.c_str();
             }
@@ -1856,8 +1871,8 @@ static string path_diffstems(const string& p1, const string& p2,
     vector<string> v1, v2;
     stringToTokens(p1, v1, "/");
     stringToTokens(p2, v2, "/");
-    unsigned int l1 = v1.size();
-    unsigned int l2 = v2.size();
+    auto l1 = v1.size();
+    auto l2 = v2.size();
         
     // Search for common leaf part
     unsigned int cl = 0;
@@ -2101,8 +2116,8 @@ string RclConfig::findFilter(const string &icmd) const
     temp = path_cat(m->m_datadir, "filters");
     temp = path_cat(temp, "python");
     PATH = temp + path_PATHsep() + PATH;
-    PATH = path_thisexecpath() + path_PATHsep() + PATH;
 #endif
+    PATH = path_thisexecdir() + path_PATHsep() + PATH;
     // Prepend possible configuration parameter?
     if (getConfParam(string("filtersdir"), temp)) {
         temp = path_tildexpand(temp);
@@ -2147,17 +2162,6 @@ bool RclConfig::processFilterCmd(std::vector<std::string>& cmd) const
     return true;
 }
 
-// This now does nothing more than processFilterCmd (after we changed to relying on the py extension)
-bool RclConfig::pythonCmd(const std::string& scriptname, std::vector<std::string>& cmd) const
-{
-#ifdef _WIN32
-    cmd = {scriptname};
-#else
-    cmd = {scriptname};
-#endif
-    return processFilterCmd(cmd);
-}
-
 /** 
  * Return decompression command line for given mime type
  */
